"use strict";
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);
var __publicField = (obj, key, value) => {
  __defNormalProp(obj, typeof key !== "symbol" ? key + "" : key, value);
  return value;
};

// bitbazaar/log/index.ts
var log_exports = {};
__export(log_exports, {
  GlobalLog: () => GlobalLog,
  LOG: () => LOG
});
module.exports = __toCommonJS(log_exports);

// bitbazaar/log/log.ts
var import_context_zone = require("@opentelemetry/context-zone");
var import_sdk_trace_web = require("@opentelemetry/sdk-trace-web");
var import_core = require("@opentelemetry/core");
var import_instrumentation_document_load = require("@opentelemetry/instrumentation-document-load");
var import_instrumentation_fetch = require("@opentelemetry/instrumentation-fetch");
var import_instrumentation_user_interaction = require("@opentelemetry/instrumentation-user-interaction");
var import_resources = require("@opentelemetry/resources");
var import_semantic_conventions = require("@opentelemetry/semantic-conventions");
var import_instrumentation = require("@opentelemetry/instrumentation");
var import_sdk_trace_base = require("@opentelemetry/sdk-trace-base");
var import_sdk_logs = require("@opentelemetry/sdk-logs");
var import_exporter_logs_otlp_http = require("@opentelemetry/exporter-logs-otlp-http");
var import_exporter_metrics_otlp_http = require("@opentelemetry/exporter-metrics-otlp-http");
var import_exporter_trace_otlp_http = require("@opentelemetry/exporter-trace-otlp-http");
var import_sdk_metrics = require("@opentelemetry/sdk-metrics");
var _LOG;
var GlobalLog = class {
  /** Create the global logger, must setup oltp (http), console can be optionally setup and will just print logs. */
  constructor({
    otlp,
    console: console2 = void 0
  }) {
    __publicField(this, "loggerProvider");
    __publicField(this, "logger");
    __publicField(this, "tracerProvider");
    __publicField(this, "tracer");
    __publicField(this, "meterProvider");
    __publicField(this, "console");
    __publicField(this, "oltp");
    /// We override global console functions to do filtering and emit to oltp, need to keep access to the inner ones:
    __publicField(this, "orig_console_fns");
    this.console = console2 ? console2 : null;
    this.oltp = otlp;
    this._set_console_fns();
    const resource = new import_resources.Resource({
      [import_semantic_conventions.SemanticResourceAttributes.SERVICE_NAME]: otlp.service_name,
      [import_semantic_conventions.SemanticResourceAttributes.SERVICE_VERSION]: otlp.service_version
    });
    const baseExporterConfig = {
      keepAlive: true
    };
    const bufferConfig = {
      maxQueueSize: 2048,
      maxExportBatchSize: 512,
      scheduledDelayMillis: 5e3,
      exportTimeoutMillis: 3e4
    };
    this.meterProvider = new import_sdk_metrics.MeterProvider({
      resource,
      readers: [
        new import_sdk_metrics.PeriodicExportingMetricReader({
          exporter: new import_exporter_metrics_otlp_http.OTLPMetricExporter({
            ...baseExporterConfig,
            url: `${otlp.endpoint}/v1/metrics`
          }),
          exportIntervalMillis: 6e4
          // Haven't found a way for it to not send when no metrics yet, so changing from 1s to 60s to not bloat the network logs of a client.
        })
      ]
    });
    this.loggerProvider = new import_sdk_logs.LoggerProvider({
      resource
    });
    this.loggerProvider.addLogRecordProcessor(
      new import_sdk_logs.BatchLogRecordProcessor(
        new import_exporter_logs_otlp_http.OTLPLogExporter({
          ...baseExporterConfig,
          url: `${otlp.endpoint}/v1/logs`
        })
      )
    );
    this.logger = this.loggerProvider.getLogger("GlobalLog");
    this.tracerProvider = new import_sdk_trace_web.WebTracerProvider({ resource });
    this.tracerProvider.addSpanProcessor(
      new import_sdk_trace_base.BatchSpanProcessor(
        new import_exporter_trace_otlp_http.OTLPTraceExporter({
          ...baseExporterConfig,
          url: `${otlp.endpoint}/v1/traces`
        }),
        bufferConfig
      )
    );
    this.tracerProvider.register({
      contextManager: new import_context_zone.ZoneContextManager(),
      // Configure the propagator to enable context propagation between services,
      // uses the W3C Trace Headers (traceparent, tracestate) and W3C Baggage Headers (baggage).
      propagator: new import_core.CompositePropagator({
        propagators: [new import_core.W3CBaggagePropagator(), new import_core.W3CTraceContextPropagator()]
      })
    });
    this.tracer = this.tracerProvider.getTracer("GlobalLog");
    (0, import_instrumentation.registerInstrumentations)({
      instrumentations: [
        // Trace user site interactions:
        new import_instrumentation_user_interaction.UserInteractionInstrumentation({}),
        // Trace client document loading:
        new import_instrumentation_document_load.DocumentLoadInstrumentation({}),
        // Auto instrument fetch requests:
        new import_instrumentation_fetch.FetchInstrumentation({})
      ]
    });
    _LOG = this;
  }
  /**
   * Get a new Meter instance to record metrics with.
   *
   * @example
   *  ```typescript
   * const meter = globalLog.getMeter("example-meter");
   * const counter = meter.createCounter('metric_name');
   * counter.add(10, { 'key': 'value' });
   * ```
   */
  meter(name, opts = {}) {
    return this.meterProvider.getMeter(
      name,
      typeof opts === "object" && "version" in opts ? opts.version : void 0,
      opts
    );
  }
  /**
   * Run a sync callback inside a span.
   */
  withSpan(name, cb) {
    return this.tracer.startActiveSpan(name, (span) => {
      const result = cb(span);
      span.end();
      return result;
    });
  }
  /**
   * Run an async callback inside a span.
   */
  withSpanAsync(name, cb) {
    return this.tracer.startActiveSpan(name, (span) => {
      return cb(span).then((result) => {
        span.end();
        return result;
      });
    });
  }
  /** Log a debug message. */
  debug(message, ...attrs) {
    this._log_inner("DEBUG", message, ...attrs);
  }
  /** Log an info message. */
  info(message, ...attrs) {
    this._log_inner("INFO", message, ...attrs);
  }
  /** Log a warning message. */
  warn(message, ...attrs) {
    this._log_inner("WARN", message, ...attrs);
  }
  /** Log an error message. */
  error(message, ...attrs) {
    this._log_inner("ERROR", message, ...attrs);
  }
  _joined_msg_and_attrs(message, attrs) {
    if (attrs.length === 0) {
      return message;
    }
    return `${message} ${attrs.join(" ")}`;
  }
  _log_inner(severityText, message, ...attrs) {
    if (this.console) {
      let emit = false;
      let emitter;
      switch (this.console.level_from) {
        case "DEBUG": {
          emit = true;
          emitter = this.orig_console_fns.debug;
          break;
        }
        case "INFO": {
          emit = severityText !== "DEBUG";
          emitter = this.orig_console_fns.info;
          break;
        }
        case "WARN": {
          emit = severityText === "WARN" || severityText === "ERROR";
          emitter = this.orig_console_fns.warn;
          break;
        }
        case "ERROR": {
          emit = severityText === "ERROR";
          emitter = this.orig_console_fns.error;
          break;
        }
      }
      if (emit) {
        if (this.console.custom_out) {
          this.console.custom_out(this._joined_msg_and_attrs(message, attrs));
        } else {
          emitter(message);
        }
      }
    }
    let emitOltp = false;
    switch (this.oltp.level_from) {
      case "DEBUG": {
        emitOltp = true;
        break;
      }
      case "INFO": {
        emitOltp = severityText !== "DEBUG";
        break;
      }
      case "WARN": {
        emitOltp = severityText === "WARN" || severityText === "ERROR";
        break;
      }
      case "ERROR": {
        emitOltp = severityText === "ERROR";
        break;
      }
    }
    if (emitOltp) {
      this.logger.emit({
        severityText,
        body: this._joined_msg_and_attrs(message, attrs)
      });
    }
  }
  _set_console_fns() {
    this.orig_console_fns = {
      log: console.log,
      debug: console.debug,
      info: console.info,
      warn: console.warn,
      error: console.error
    };
    console.log = (msg, ...attrs) => this.info(msg, ...attrs);
    console.debug = (msg, ...attrs) => this.debug(msg, ...attrs);
    console.info = (msg, ...attrs) => this.info(msg, ...attrs);
    console.warn = (msg, ...attrs) => this.warn(msg, ...attrs);
    console.error = (msg, ...attrs) => this.error(msg, ...attrs);
  }
};
var LOG = new Proxy(GlobalLog.prototype, {
  get: (target, property, receiver) => {
    if (_LOG) {
      return Reflect.get(_LOG, property, receiver);
    }
    throw new Error("Global log not yet initialized!");
  }
});
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  GlobalLog,
  LOG
});
