"use strict";
var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toESM = (mod, isNodeMode, target) => (target = mod != null ? __create(__getProtoOf(mod)) : {}, __copyProps(
  // If the importer is in node compatibility mode or this is not an ESM
  // file that has been converted to a CommonJS file using a Babel-
  // compatible transform (i.e. "__esModule" has not been set), then set
  // "default" to the CommonJS "module.exports" for node compatibility.
  isNodeMode || !mod || !mod.__esModule ? __defProp(target, "default", { value: mod, enumerable: true }) : target,
  mod
));
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// bitbazaar/vite/index.ts
var vite_exports = {};
__export(vite_exports, {
  createConfig: () => createConfig
});
module.exports = __toCommonJS(vite_exports);

// bitbazaar/vite/createConfig.ts
var import_preset_vite = __toESM(require("@preact/preset-vite"), 1);
var import_html_minifier_terser = require("html-minifier-terser");
var import_vite = require("vite");
var import_vite_plugin_checker = __toESM(require("vite-plugin-checker"), 1);
var import_vite_plugin_inspect = __toESM(require("vite-plugin-inspect"), 1);
var import_vite_plugin_pwa = require("vite-plugin-pwa");

// bitbazaar/utils/genPath.ts
var genPath = (path_in, {
  sShlash = void 0,
  // If isn't a root url (e.g. starting with http), or a relative path, then will default to true, otherwise false
  eSlash = void 0,
  // NOTE! defaults to true if dir, false if file
  extra = void 0
  // Extra sections to add to the end of the path, the sSlash/eSlash applies after this has been added:
} = {}) => {
  let path = path_in.trim();
  if (extra) {
    extra = extra.map((e) => e.trim());
  }
  if (extra) {
    if (!path.endsWith("/")) {
      path = `${path}/`;
    }
    extra.forEach((e) => {
      path = `${path}${e}/`;
    });
  }
  if (sShlash === void 0) {
    sShlash = !path.startsWith("http") && !path.startsWith(".");
  }
  if (eSlash === void 0) {
    const tmpPath = path.endsWith("/") ? path.slice(0, -1) : path;
    const isFile = tmpPath.split("/").pop()?.includes(".");
    eSlash = !isFile;
  }
  if (sShlash && !path.startsWith("/")) {
    path = `/${path}`;
  } else if (!sShlash && path.startsWith("/")) {
    path = path.slice(1);
  }
  if (eSlash && !path.endsWith("/")) {
    path = `${path}/`;
  } else if (!eSlash && path.endsWith("/")) {
    path = path.slice(0, -1);
  }
  return path;
};

// bitbazaar/vite/createConfig.ts
var import_promises = __toESM(require("fs/promises"), 1);

// bitbazaar/vite/genProxy.ts
var genTargetOptions = (target) => {
  return {
    // Make sure DOES NOT end with /: (if it has one it won't forward sub paths)
    target: genPath(target, { eSlash: false }),
    // If true, django csrf breaks I think because csrf comes in as port 3000,
    // but vites rewritten headers as django's port 8000,
    // seems like there aren't any issues in dev with just leaving as vite domain.
    changeOrigin: false
  };
};
var genBackendProxies = ({ target, matches, negate }) => {
  const result = {};
  if (negate) {
    result[`^(?!${matches.join("|")}).*$`] = genTargetOptions(target);
  } else {
    matches.forEach((path) => {
      result[path] = genTargetOptions(target);
    });
  }
  return result;
};

// bitbazaar/vite/createConfig.ts
var baseNonFrontendGlobs = [
  "**/.git/**",
  "**/venv/**",
  "**/.venv/**",
  "**/node_modules/**",
  "**/dist/**",
  "**/lib/**",
  "**/cypress/**",
  "**/coverage/**",
  "**/.eslintrc.*/**",
  "**/*.etch.*",
  "**/.{idea,git,cache,output,temp,mypy_cache,pytype,pytest,pyright}/**",
  "**/{karma,rollup,webpack,vite,vitest,jest,ava,babel,nyc,cypress,tsup,build}.config.*"
];
var pathExists = async (path) => {
  try {
    await import_promises.default.access(path);
    return true;
  } catch {
    return false;
  }
};
var createConfig = (mode, conf) => {
  const isProd = mode === "production";
  const isTest = mode === "test";
  const isDev = mode === "development";
  if (!isProd && !isTest && !isDev) {
    throw new Error(`Unexpected vite mode: ${mode}`);
  }
  console.log(`Vite mode: ${mode}, prod=${isProd}, test=${isTest}, dev=${isDev}`);
  const assetsPath = genPath(conf.staticPath, {
    extra: ["dist"]
  });
  const assetsUrl = genPath(conf.staticUrl, {
    extra: ["dist"]
  });
  const nonFrontendGlobs = [...baseNonFrontendGlobs, ...conf.extraNonFrontendGlobs || []];
  const plugins = [
    (0, import_preset_vite.default)({
      devToolsEnabled: !isProd,
      prefreshEnabled: !isProd
    }),
    // The service worker / pwa stuff:
    (0, import_vite_plugin_pwa.VitePWA)({
      registerType: "autoUpdate",
      workbox: {
        // Precache everything found in the vite dist folder:
        globDirectory: genPath(assetsPath),
        // Excluding html, we only have one root index.html and want that to always be fresh:
        globPatterns: ["**/*.{js,css,ico,png,svg}"],
        // Don't fallback on document based (e.g. `/some-page`) requests
        // Even though this says `null` by default, I had to set this specifically to `null` to make it work
        navigateFallback: null,
        // Tell the worker it should be finding all the files from the static domain, rather than the backend:
        modifyURLPrefix: {
          "": genPath(assetsUrl)
        }
      },
      // Needs to come from the backend/same domain otherwise CORS won't work (classic worker problems)
      srcDir: genPath(assetsPath),
      outDir: genPath(conf.sameDomStaticPath, {
        extra: ["sworker"]
      }),
      // Note the mainfest.webmanifest is incorrectly placed in static/dist, but the links go the backend url.
      // Bug in the lib, haven't found a proper way to solve.
      // A secondary plugin below moves the file at the end.
      base: genPath(conf.sameDomStaticUrl, {
        extra: ["sworker"]
      }),
      buildBase: genPath(conf.sameDomStaticUrl, {
        extra: ["sworker"]
      }),
      // Puts the sw importer script directly in the index.html head, rather than a separate file:
      injectRegister: "inline",
      // Allowing the service worker to control the full stie rather than the dir it comes from:
      // this requires the Service-Worker-Allowed: "/" header to be passed from the worker serve directory on backend (which it is)
      scope: "/",
      manifest: {
        name: conf.siteName,
        short_name: conf.siteName,
        description: conf.siteDescription,
        theme_color: "#031033",
        background_color: "#031033",
        display: "standalone",
        start_url: "/",
        icons: [
          {
            src: conf.favicon192PngPath,
            sizes: "192x192",
            type: "image/png",
            purpose: "any maskable"
          },
          {
            src: conf.favicon512PngPath,
            sizes: "512x512",
            type: "image/png",
            purpose: "any maskable"
          }
        ]
      }
    }),
    {
      name: "move-webmanifest",
      // the name of your custom plugin. Could be anything.
      apply: "build",
      enforce: "post",
      closeBundle: async () => {
        const oldLoc = genPath(assetsPath, {
          extra: ["manifest.webmanifest"]
        });
        const newLoc = genPath(conf.sameDomStaticPath, {
          extra: ["sworker", "manifest.webmanifest"]
        });
        if (!await pathExists(genPath(conf.sameDomStaticPath, { extra: ["sworker"] }))) {
          await import_promises.default.mkdir(genPath(conf.sameDomStaticPath, { extra: ["sworker"] }));
        }
        console.log(`Moving webmanifest from ${oldLoc} to ${newLoc}`);
        await import_promises.default.rename(oldLoc, newLoc);
        const indexLoc = genPath(assetsPath, {
          extra: ["index.html"]
        });
        console.log(`Reading index.html from ${oldLoc} and minifying...`);
        const indexSrc = await import_promises.default.readFile(indexLoc, "utf-8");
        const indexMinified = await (0, import_html_minifier_terser.minify)(indexSrc, {
          collapseBooleanAttributes: true,
          collapseInlineTagWhitespace: true,
          collapseWhitespace: true,
          useShortDoctype: true,
          removeComments: true,
          minifyCSS: true,
          minifyJS: true
        });
        const newIndexLoc = genPath(conf.sameDomStaticPath, {
          extra: ["site_index.html"]
        });
        console.log(
          `Minified index.html from ${indexSrc.length} to ${indexMinified.length} bytes, writing to ${newIndexLoc}.`
        );
        await import_promises.default.writeFile(newIndexLoc, indexMinified);
        await import_promises.default.rm(indexLoc);
      }
    }
  ];
  if (isDev && conf.inspect) {
    plugins.push((0, import_vite_plugin_inspect.default)());
  }
  if (!isTest) {
    plugins.push(
      (0, import_vite_plugin_checker.default)({
        typescript: true,
        enableBuild: true,
        // Want it to validate type checking on build too
        // Show errors in terminal & UI:
        terminal: true,
        overlay: true
      })
    );
  }
  const config = {
    clearScreen: false,
    // Don't clear screen when running vite commands
    plugins,
    resolve: {
      // Providing absolute paths:
      alias: conf.alias,
      // Fixes esm problem with rrd:
      // react-router-dom specifies "module" field in package.json for ESM entry
      // if it's not mapped, it uses the "main" field which is CommonJS that redirects to CJS preact
      mainFields: ["module"]
    },
    server: {
      // open: true, // Open the web page on start
      host: "localhost",
      port: conf.port,
      watch: {
        // Don't check non frontend files:
        ignored: nonFrontendGlobs
      },
      // Proxy all backend requests to django/fastapi:
      proxy: conf.proxy ? genBackendProxies(conf.proxy) : void 0,
      fs: conf.serverFs
    },
    // When being served from django in production, js internal assets urls need to use url to the js assets inside the static url,
    // rather than in dev where they're just served from the root:
    base: genPath(isProd ? assetsUrl : "/"),
    // The production build config:
    build: {
      outDir: genPath(assetsPath),
      // Disabled by default when outside root, just a precautionary check I'm happy to disable:
      emptyOutDir: true
    },
    define: {
      // Vite doesn't have process, so define the specifically needed ones directly:
      "process.env.NODE_ENV": JSON.stringify(mode)
    },
    // (Attempted!) size and perf optimisations:
    json: {
      stringify: true
    },
    esbuild: {
      exclude: nonFrontendGlobs
    }
  };
  return (0, import_vite.defineConfig)(config);
};
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  createConfig
});
