"use strict";
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// bitbazaar/wouter/index.ts
var wouter_exports = {};
__export(wouter_exports, {
  route: () => route,
  routeParams: () => routeParams,
  routes: () => routes
});
module.exports = __toCommonJS(wouter_exports);

// bitbazaar/wouter/routes.tsx
var import_wouter = require("wouter");

// bitbazaar/wouter/valiWrapper.tsx
var import_react = require("react");
var import_jsx_runtime = require("react/jsx-runtime");
var ValiWrapper = ({
  params,
  validators,
  Comp,
  Comp404
}) => {
  const cleanedParams = (0, import_react.useMemo)(() => {
    try {
      const cleaned = {};
      for (const [key, value] of Object.entries(params)) {
        const validator = validators[key];
        if (!validator) {
          return null;
        }
        cleaned[key] = validator(value);
      }
      return cleaned;
    } catch (e) {
      return null;
    }
  }, [validators, params]);
  if (cleanedParams === null) {
    return /* @__PURE__ */ (0, import_jsx_runtime.jsx)(Comp404, {});
  }
  return /* @__PURE__ */ (0, import_jsx_runtime.jsx)(Comp, { ...cleanedParams });
};

// bitbazaar/wouter/routes.tsx
var import_jsx_runtime2 = require("react/jsx-runtime");
var routes = ({
  Comp404,
  routes: routes2
}) => {
  const output = {};
  Object.entries(routes2).forEach(([key, props]) => {
    output[key] = {
      ...props,
      route: /* @__PURE__ */ (0, import_jsx_runtime2.jsx)(import_wouter.Route, { path: props.path, children: (params) => /* @__PURE__ */ (0, import_jsx_runtime2.jsx)(
        ValiWrapper,
        {
          params,
          validators: props.validators,
          Comp: props.Comp,
          Comp404
        }
      ) }, props.path)
    };
  });
  const allRoutes = Object.values(output).map(({ route: route2 }) => route2);
  return {
    switch: /* @__PURE__ */ (0, import_jsx_runtime2.jsxs)(import_wouter.Switch, { children: [
      allRoutes,
      /* @__PURE__ */ (0, import_jsx_runtime2.jsx)(import_wouter.Route, { component: Comp404 })
    ] }),
    // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment
    paths: output
  };
};

// bitbazaar/utils/genPath.ts
var genPath = (path_in, {
  sShlash = void 0,
  // If isn't a root url (e.g. starting with http), or a relative path, then will default to true, otherwise false
  eSlash = void 0,
  // NOTE! defaults to true if dir, false if file
  extra = void 0
  // Extra sections to add to the end of the path, the sSlash/eSlash applies after this has been added:
} = {}) => {
  let path = path_in.trim();
  if (extra) {
    extra = extra.map((e) => e.trim());
  }
  if (extra) {
    if (!path.endsWith("/")) {
      path = `${path}/`;
    }
    extra.forEach((e) => {
      path = `${path}${e}/`;
    });
  }
  if (sShlash === void 0) {
    sShlash = !path.startsWith("http") && !path.startsWith(".");
  }
  if (eSlash === void 0) {
    const tmpPath = path.endsWith("/") ? path.slice(0, -1) : path;
    const isFile = tmpPath.split("/").pop()?.includes(".");
    eSlash = !isFile;
  }
  if (sShlash && !path.startsWith("/")) {
    path = `/${path}`;
  } else if (!sShlash && path.startsWith("/")) {
    path = path.slice(1);
  }
  if (eSlash && !path.endsWith("/")) {
    path = `${path}/`;
  } else if (!eSlash && path.endsWith("/")) {
    path = path.slice(0, -1);
  }
  return path;
};

// bitbazaar/utils/index.ts
var areTestsRunning = process.env.NODE_ENV === "test";
var assertNever = (neverVal) => {
  throw new Error(
    // eslint-disable-next-line @typescript-eslint/restrict-template-expressions
    `Internal error. This should never have happened! Invalid neverValue: "${neverVal}".`
  );
};

// bitbazaar/wouter/route.ts
var route = ({
  base,
  params,
  Comp
}) => {
  base = genPath(base, { sShlash: true, eSlash: true });
  let path = base;
  params.forEach(([param_name]) => {
    path += `:${param_name}/`;
  });
  if (path !== "/") {
    path = path.slice(0, -1);
  }
  const validators = {};
  params.forEach(([param_name, param_type]) => {
    let validator = null;
    switch (param_type) {
      case "float": {
        validator = (param) => {
          const num = parseFloat(param);
          if (Number.isNaN(num)) {
            throw new Error("Invalid number");
          }
          return num;
        };
        break;
      }
      case "int": {
        validator = (param) => {
          const num = parseInt(param);
          if (Number.isNaN(num)) {
            throw new Error("Invalid number");
          }
          return num;
        };
        break;
      }
      case "boolean": {
        validator = (param) => {
          const lower = param.toLowerCase();
          if (["true", "True", "1", "y"].includes(lower)) {
            return true;
          }
          if (["false", "False", "0", "n"].includes(lower)) {
            return false;
          }
          throw new Error("Invalid boolean");
        };
        break;
      }
      case "string": {
        validator = (param) => param;
        break;
      }
      default:
        assertNever(param_type);
    }
    validators[param_name] = validator;
  });
  const routeObj = {
    build: (buildParams) => {
      let builtPath = path;
      params.forEach(([param_name]) => {
        builtPath = builtPath.replace(`:${param_name}`, buildParams[param_name]);
      });
      return builtPath;
    },
    validators,
    Comp,
    path
  };
  return routeObj;
};

// bitbazaar/wouter/routeParams.ts
var routeParams = (params) => {
  return params;
};
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  route,
  routeParams,
  routes
});
