import {
  assertNever
} from "../chunk-KQRCISBG.js";
import {
  genPath
} from "../chunk-CQILZEMP.js";
import "../chunk-XEO3YXBM.js";

// bitbazaar/wouter/routes.tsx
import { Route, Switch } from "wouter";

// bitbazaar/wouter/valiWrapper.tsx
import { useMemo } from "react";
import { jsx } from "react/jsx-runtime";
var ValiWrapper = ({
  params,
  validators,
  Comp,
  Comp404
}) => {
  const cleanedParams = useMemo(() => {
    try {
      const cleaned = {};
      for (const [key, value] of Object.entries(params)) {
        const validator = validators[key];
        if (!validator) {
          return null;
        }
        cleaned[key] = validator(value);
      }
      return cleaned;
    } catch (e) {
      return null;
    }
  }, [validators, params]);
  if (cleanedParams === null) {
    return /* @__PURE__ */ jsx(Comp404, {});
  }
  return /* @__PURE__ */ jsx(Comp, { ...cleanedParams });
};

// bitbazaar/wouter/routes.tsx
import { jsx as jsx2, jsxs } from "react/jsx-runtime";
var routes = ({
  Comp404,
  routes: routes2
}) => {
  const output = {};
  Object.entries(routes2).forEach(([key, props]) => {
    output[key] = {
      ...props,
      route: /* @__PURE__ */ jsx2(Route, { path: props.path, children: (params) => /* @__PURE__ */ jsx2(
        ValiWrapper,
        {
          params,
          validators: props.validators,
          Comp: props.Comp,
          Comp404
        }
      ) }, props.path)
    };
  });
  const allRoutes = Object.values(output).map(({ route: route2 }) => route2);
  return {
    switch: /* @__PURE__ */ jsxs(Switch, { children: [
      allRoutes,
      /* @__PURE__ */ jsx2(Route, { component: Comp404 })
    ] }),
    // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment
    paths: output
  };
};

// bitbazaar/wouter/route.ts
var route = ({
  base,
  params,
  Comp
}) => {
  base = genPath(base, { sShlash: true, eSlash: true });
  let path = base;
  params.forEach(([param_name]) => {
    path += `:${param_name}/`;
  });
  if (path !== "/") {
    path = path.slice(0, -1);
  }
  const validators = {};
  params.forEach(([param_name, param_type]) => {
    let validator = null;
    switch (param_type) {
      case "float": {
        validator = (param) => {
          const num = parseFloat(param);
          if (Number.isNaN(num)) {
            throw new Error("Invalid number");
          }
          return num;
        };
        break;
      }
      case "int": {
        validator = (param) => {
          const num = parseInt(param);
          if (Number.isNaN(num)) {
            throw new Error("Invalid number");
          }
          return num;
        };
        break;
      }
      case "boolean": {
        validator = (param) => {
          const lower = param.toLowerCase();
          if (["true", "True", "1", "y"].includes(lower)) {
            return true;
          }
          if (["false", "False", "0", "n"].includes(lower)) {
            return false;
          }
          throw new Error("Invalid boolean");
        };
        break;
      }
      case "string": {
        validator = (param) => param;
        break;
      }
      default:
        assertNever(param_type);
    }
    validators[param_name] = validator;
  });
  const routeObj = {
    build: (buildParams) => {
      let builtPath = path;
      params.forEach(([param_name]) => {
        builtPath = builtPath.replace(`:${param_name}`, buildParams[param_name]);
      });
      return builtPath;
    },
    validators,
    Comp,
    path
  };
  return routeObj;
};

// bitbazaar/wouter/routeParams.ts
var routeParams = (params) => {
  return params;
};
export {
  route,
  routeParams,
  routes
};
